// $Id$

#include <stdlib.h>
#include <mpi.h>
#include <iostream.h>
#include "DAGH.h"
#include "DAGHIO.h"

#include "Consts.h"

//#include "fortran.h"
#include "ADMvar.h"
#include "SArray.h"
#include "TArray.h"
#include "BHmask.h"
#include "Variables.h"

#include "Berger_Oliger.h"
#include "InitData0.h"
#include "input_params.h"

int  Stop_Criterion(V &VA, int rank);

void TimeStep(GridHierarchy & GH,const INTEGER Level,
                   const DOUBLE Delta_t, DOUBLE Time,
                   V & VA, INTEGER t, INTEGER myrank);

void DataOut0(V & VA, double Time, double Delta_t, int t, int Level, int IDENT,
             int myrank);

void main(int argc, char *argv[])
{

  double testvalue;
  int nprocs,myrank;
  double starttime,endtime;
  int stopnow;
  int i;

  //  if(argc<2) {
  //    cout << "Oops: No input file specified" << endl;
  //    cout << "Please resubmit this program with a file name as an argument" 
  //            << endl;
  //   exit(1);
  //  };

  MPI_Init( &argc, &argv ); // Initialize MPI
  MPI_Comm_size(MPI_COMM_WORLD,&nprocs);
  MPI_Comm_rank(MPI_COMM_WORLD,&myrank);

  if(myrank==0) cout << " ==================================" << endl;
  if(myrank==0) cout <<  " Using " << nprocs << " processor(s)!" << endl;
  if(myrank==0) cout << " ==================================" << endl;

   double params[101];
   
   //   read_parameters(argv[1], params, myrank);
 

   //   if(myrank==0) {

    ifstream infile;
    //    infile.open(argv[1]);
    infile.open("adm.in");
    if(!infile) {
       cerr << "Can't open " << argv << " for input." << endl;
       MPI_Abort(MPI_COMM_WORLD,1);
       exit(1);
    };

    const int size =1024;
    char buf[size];
    int ipar=0;
    double parval=0;

    while(ipar<100) {  

      infile >> parval;
      infile >> ipar;
      infile.get(buf,size,'\n'); 
      //if(myrank==0) cout <<  ipar << " " << parval << " " << buf <<  endl;
      params[ipar-1]=parval;
    };

    infile.close(); 

    //   };

  params[P_NPROCS]=nprocs*1.0;  
  starttime=MPI_Wtime(); 

  if(myrank==0) cout << " This is a good begining " << endl;
    INTEGER shape[DIM];
    DOUBLE  b_box[2*DIM];
    shape[0] = (int)params[0]-1; b_box[0] = params[3]; b_box[1] = params[4];
    shape[1] = (int)params[1]-1; b_box[2] = params[5]; b_box[3] = params[6];
    shape[2] = (int)params[2]-1; b_box[4] = params[7]; b_box[5] = params[8];

    const double h1 = (b_box[1] - b_box[0])/(shape[0]);
    const double h2 = (b_box[3] - b_box[2])/(shape[1]);
    const double h3 = (b_box[5] - b_box[4])/(shape[2]);

    if(myrank==0) cout << "h1 " << h1 << " h2 " << h2 << " h3 " << h3 <<endl;

    double h = (h1 < h2) ? h1 : h2; h = (h < h3) ? h : h3;
    double Delta_T = params[P_COURANT]*h;

    params[P_GRIDSPACING]=h;
    params[P_TIMESTEP]=Delta_T;
    params[P_CURRENTTIME]=0;

    if(myrank==0) {
        cout << "start DAGH" << endl;
        cout << "spatial resolution " << params[97] << endl;
        cout << "tempral resolution " << params[98] << endl;
        cout << "hole radius " << params[23] << endl;
    }

   const int RefineFactor = 2;
   const int MaxLev = 1;
   GridHierarchy GH(DIM,DAGHNonCellCentered,MaxLev);

   SetBaseGrid(GH,b_box,shape);
   SetRefineFactor(GH,RefineFactor);
   SetBoundaryType(GH,DAGHBoundaryRegular);
   ComposeHierarchy(GH);

   //myrank = MY_PROC(GH);

   cout << "Hey! I AM PROCESSOR " << myrank << endl<< flush ;
   
   if(myrank==0) cout << " Grid Hierarchy Defined " << endl;

   const int s_sten=(int)params[P_SPATIAL_STENCIL];
   if(myrank==0) cout << "Space Stensil " << s_sten << endl;

#include "GFunctions.h"

   // placeholder for 
   //   nst = int(params(P_AH_NST))
   //  nsp = int(params(P_AH_NSP))
   //  allocate(s(nst,nsp,3),sc(nst,nsp,3))


     if(myrank==0) cout << "grid functions allocated" << endl;

#include "ol_parser.h"

     int Level = 0;
     int t = 0;
     SetCurrentTime(GH,t,Level,DAGH_Main);
     int dt=GH.timestep(Level,DAGH_Main);
     
     if(myrank==0) cout << "SetCurrentTime - done" << endl;
    
     //     int time_step = TimeStep(GH,Level,DAGH_Main); 
     cout << "call initdata0" << endl; 


    InitData0(VA,starttime,Delta_T, t, Level, DAGH_Main, myrank);

     cout << "return initdata0" << endl; 

    t=0;
    GH.setCurrentTime(t,Level,DAGH_Main);
    int iter=0;

#include "ol_output.h"     

    if(myrank==0) cout << "Beginning time index: " << t << endl;
    while(iter < (int)params[P_NSTEPS]) {

      if(myrank==0) cout << "iteration " << iter << " of " << (int)params[P_NSTEPS]-1
	   << endl << flush;

        TimeStep(GH, Level, Delta_T,starttime+(iter+1)*Delta_T, 
                 VA,iter,myrank);

      iter++;
      //      t=iter;  //replaced by puting iter to TimeStep call

#include "ol_output.h"     
      
      stopnow = Stop_Criterion(VA, myrank);
      if(stopnow == 1) break;
    };


    endtime=MPI_Wtime();
    if(myrank==0) cout << "execution time " << endtime-starttime << 
		  " seconds" << endl;
    
    if(myrank==0)  cout << "finalize MPI" << endl;

#include "ol_finalize.h"

    DAGHMPI_Finalize();
    if(myrank==0) cout << "done!" << endl;

  
}











